/* -*- c++ -*- */

/*

Copyright (c) 2015 V.Ayllon.  All rights reserved.

TwinTeethFirmware is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
to anyone for the consequences of using it or for whether it serves any
particular purpose or works at all, unless he says so in writing.  Refer
to the GNU General Public License, Version 2 or later, for full details.

Everyone is granted permission to copy, modify and redistribute
TwinTeeth Firmware, but only under the conditions described in the GNU General Public
License.  A copy of this license is supposed to have been
given to you along with TwinTeeth Firmware so you can know your rights and
responsibilities.  It should be in a file named COPYING.  Among other
things, the copyright notice and this notice must be preserved on all
copies.

Based on Sprinter and grbl.
Copyright (C) 2011 Camiel Gubbels / Erik van der Zalm

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.


This firmware is a mashup between Sprinter and grbl.
(https://github.com/kliment/Sprinter)
(https://github.com/simen/grbl/tree)

It has preliminary support for Matthew Roberts advance algorithm
http://reprap.org/pipermail/reprap-dev/2011-May/003323.html
*/

#include "TwinTeeth.h"

#ifdef ENABLE_AUTO_BED_LEVELING
//#include "vector_3.h"
#endif // ENABLE_AUTO_BED_LEVELING

//#include "ultralcd.h"

#include "stepper.h"
#include "temperature.h"
#include "motion_control.h"
#include "ConfigurationStore.h"
#include "language.h"
#include "pins_arduino.h"
#include "math.h"
#include "laser_control.h"
#include "focus.h"
#include "Base64.h"
#include "backlash.h"



#define VERSION_STRING  "1.0.0"

// look here for descriptions of gcodes: http://linuxcnc.org/handbook/gcode/g-code.html
// http://objects.reprap.org/wiki/Mendel_User_Manual:_RepRapGCodes

//Implemented Codes
//-------------------
// G0  -> G1
// G1  - Coordinated Movement X Y Z E
// G2  - CW ARC
// G3  - CCW ARC
// G4  - Dwell S<seconds> or P<milliseconds>
// G7  - Raster mode (Laser tool only)
// G10 - retract filament according to settings of M207
// G11 - retract recover filament according to settings of M208
// G28 - Home all Axis
// G29 - Detailed Z-Probe, probes the bed at multiples points. <T > Test the autoleveling system
// G30 - Single Z Probe, probes bed at current XY location.
// G31 - Focus laser (Laser tool only) <O offset> <W width> <H height> <T test mode> - W and H measures in points not mm. O in mm.
// G32 - Init raster (Laser tool only) <D diode> <W width> <H height>
// G54 - Select active tool - G54 P~
// G90 - Use Absolute Coordinates
// G91 - Use Relative Coordinates
// G92 - Set current position to cordinates given
// G93 - Set program zero offset for active tool
// G94 - Reset config data for active tool or tool specified
// G95 - Set actual Z axis min value


// M Codes
// M0   - Unconditional stop - Wait for user to press a button on the LCD (Only if ULTRA_LCD is enabled)
// M1   - Same as M0
// M02  - End of program (ignored)
// M03  - Set laser power (S Power) or Spindle rotation normal (S Speed)
// M05  - Set default laser and focus diode/Spindle off
// M06  - Turn laser on/Spindle on
// M07  - Test mode off/on use S[0 1]
// M17  - Enable/Power all stepper motors
// M18  - Disable all stepper motors; same as M84
// M20  - List SD card
// M21  - Init SD card
// M22  - Release SD card
// M23  - Select SD file (M23 filename.g)
// M24  - Start/resume SD print
// M25  - Pause SD print
// M26  - Set SD position in bytes (M26 S12345)
// M27  - Report SD print status
// M28  - Start SD write (M28 filename.g)
// M29  - Stop SD write
// M30  - Delete file from SD (M30 filename.g)
// M31  - Output time since last M109 or SD card start to serial
// M32  - Select file and start SD print (Can be used _while_ printing from SD card files):
//        syntax "M32 /path/filename#", or "M32 S<startpos bytes> !filename#"
//        Call gcode file : "M32 P !filename#" and return to caller file after finishing (simiarl to #include).
//        The '#' is necessary when calling from within sd files, as it stops buffer prereading
// M80  - Turn on Power Supply
// M81  - Turn off Power Supply
// M82  - Set E codes absolute (default)
// M83  - Set E codes relative while in Absolute Coordinates (G90) mode
// M84  - Disable steppers until next move,
//        or use S<seconds> to specify an inactivity timeout, after which the steppers will be disabled.  S0 to disable the timeout.
// M85  - Set inactivity shutdown timer with parameter S<seconds>. To disable set zero (default)
// M92  - Set axis_steps_per_unit - same syntax as G92
// M104 - Set extruder target temp
// M105 - Read current temp
// M106 - Fan on
// M107 - Fan off
// M109 - Sxxx Wait for extruder current temp to reach target temp. Waits only when heating
//        Rxxx Wait for extruder current temp to reach target temp. Waits when heating and cooling
// M114 - Output current position to serial port
// M115 - Capabilities string
// M117 - display message
// M119 - Output Endstop status to serial port
// M126 - Solenoid Air Valve Open (BariCUDA support by jmil)
// M127 - Solenoid Air Valve Closed (BariCUDA vent to atmospheric pressure by jmil)
// M128 - EtoP Open (BariCUDA EtoP = electricity to air pressure transducer by jmil)
// M129 - EtoP Closed (BariCUDA EtoP = electricity to air pressure transducer by jmil)
// M140 - Set bed target temp
// M150 - Set BlinkM Colour Output R: Red<0-255> U(!): Green<0-255> B: Blue<0-255> over i2c, G for green does not work.
// M190 - Sxxx Wait for bed current temp to reach target temp. Waits only when heating
//        Rxxx Wait for bed current temp to reach target temp. Waits when heating and cooling
// M200 D<millimeters>- set filament diameter and set E axis units to cubic millimeters (use S0 to set back to millimeters).
// M201 - Set max acceleration in units/s^2 for print moves (M201 X1000 Y1000)
// M202 - Set max acceleration in units/s^2 for travel moves (M202 X1000 Y1000) Unused in Marlin!!
// M203 - Set maximum feedrate that your machine can sustain (M203 X200 Y200 Z300 E10000) in mm/sec
// M204 - Set default acceleration: S normal moves T filament only moves (M204 S3000 T7000) im mm/sec^2  also sets minimum segment time in ms (B20000) to prevent buffer underruns and M20 minimum feedrate
// M205 -  advanced settings:  minimum travel speed S=while printing T=travel only,  B=minimum segment time X= maximum xy jerk, Z=maximum Z jerk, E=maximum E jerk
// M206 - set additional homeing offset
// M207 - set retract length S[positive mm] F[feedrate mm/min] Z[additional zlift/hop], stays in mm regardless of M200 setting
// M208 - set recover=unretract length S[positive mm surplus to the M207 S*] F[feedrate mm/min]
// M209 - S<1=true/0=false> enable automatic retract detect if the slicer did not support G10/11: every normal extrude-only move will be classified as retract depending on the direction.
// M218 - set hotend offset (in mm): T<extruder_number> X<offset_on_X> Y<offset_on_Y>
// M220 S<factor in percent>- set speed factor override percentage
// M221 S<factor in percent>- set extrude factor override percentage
// M240 - Trigger a camera to take a photograph
// M250 - Set LCD contrast C<contrast value> (value 0..63)
// M280 - set servo position absolute. P: servo index, S: angle or microseconds
// M301 - Set PID parameters P I and D
// M302 - Allow cold extrudes, or set the minimum extrude S<temperature>.
// M303 - PID relay autotune S<temperature> sets the target temperature. (default target temperature = 150C)
// M304 - Set bed PID parameters P I and D
// M400 - Finish all moves
// M500 - stores paramters in EEPROM
// M501 - reads parameters from EEPROM (if you need reset them after you changed them temporarily).
// M502 - reverts to the default "factory settings".  You still need to store them in EEPROM afterwards if you want to.
// M503 - print the current settings (from memory not from eeprom)
// M504 - Read tool configuration from EEPROM
// M505 - Retrieve all tool configuration
// M506 - Reset tool config data
// M507 - print tool config data
// M508 - Set seek rate (rate for rapid movements with G0 command)

// M540 - Use S[0|1] to enable or disable the stop SD card print on endstop hit (requires ABORT_ON_ENDSTOP_HIT_FEATURE_ENABLED)
// M600 - Pause for filament change X[pos] Y[pos] Z[relative lift] E[initial retract] L[later retract distance for removal]
// M666 - set delta endstop adjustemnt
// M605 - Set dual x-carriage movement mode: S<mode> [ X<duplication x-offset> R<duplication temp offset> ]
// M907 - Set digital trimpot motor current using axis codes.
// M350 - Set microstepping mode.
// M351 - Toggle MS1 MS2 pins directly.
// M928 - Start SD logging (M928 filename.g) - ended by M29
// M999 - Restart after being stopped by error

//Stepper Movement Variables

//===========================================================================
//=============================imported variables============================
//===========================================================================


//===========================================================================
//=============================public variables=============================
//===========================================================================

#ifdef SDSUPPORT
CardReader card;
#endif
float homing_feedrate[] = HOMING_FEEDRATE;
bool axis_relative_modes[] = AXIS_RELATIVE_MODES;
int feedmultiply=100; //100->1 200->2
int saved_feedmultiply;
int extrudemultiply=100; //100->1 200->2
float volumetric_multiplier[EXTRUDERS] = {1.0};
float current_position[NUM_AXIS] = { 0.0, 0.0, 0.0, 0.0 };
float machine_position[NUM_AXIS] = { 0.0, 0.0, 0.0, 0.0 };
float z_min_pos[MAX_TOOL_NUMBER] = {Z_MIN_PEN_TOOL,Z_MIN_DRILL_TOOL,Z_MIN_LASER_TOOL,Z_MIN_EXTRUDER_TOOL,Z_MIN_DISPENSER_TOOL};
short current_direction[3] = {1,1,1};

float add_homeing[3]={0,0,0};
#ifdef DELTA
float endstop_adj[3]={0,0,0};
#endif
float min_pos[3] = { X_MIN_POS, Y_MIN_POS, Z_MIN_POS };
float max_pos[3] = { X_MAX_POS, Y_MAX_POS, Z_MAX_POS };
bool axis_known_position[3] = {false, false, false};
float draw_dim[2] = { 0., 0.}; 
int draw_pixels[2] = {0,0};
float zprobe_zoffset;
uint8_t active_extruder = 0;
int fanSpeed=0;
#ifdef SERVO_ENDSTOPS
int servo_endstops[] = SERVO_ENDSTOPS;
int servo_endstop_angles[] = SERVO_ENDSTOP_ANGLES;
#endif

#ifdef DELTA
float delta[3] = {0.0, 0.0, 0.0};
#endif


TOOL_CONFIG tool_config[MAX_TOOL_NUMBER];
#ifdef NONLINEAR_BED_LEVELING
bool isBed_leveled;
float bed_level[ACCURATE_BED_LEVELING_POINTS][ACCURATE_BED_LEVELING_POINTS];
#endif
short active_tool = -1;

bool raster_direction = 0;		 // Raster direction
float destination[NUM_AXIS] = {  0.0, 0.0, 0.0, 0.0};
float next_feedrate;
uint8_t focus_status = FOCUS_ST_RESET;

int buffer_pixels;
bool test_mode = false;
bool wait_flag = false;

//===========================================================================
//=============================private variables=============================
//===========================================================================
const char axis_codes[NUM_AXIS] = {'X', 'Y', 'Z', 'E'};
static float offset[3] = {0.0, 0.0, 0.0};
static bool home_all_axis = true;
static float feedrate = 1500.0, saved_feedrate, seek_rate = DEFAULT_SEEK_RATE;
//static long gcode_N, gcode_LastN, Stopped_gcode_LastN = 0;

static bool relative_mode = false;  //Determines Absolute or Relative Coordinates

static char cmdbuffer[BUFSIZE][MAX_CMD_SIZE];
static int bufindr = 0;
static int bufindw = 0;
static int buflen = 0;
//static int i = 0;
//static char serial_char;
static int serial_count = 0;
static char comment_mode = NULL; 
static char *strchr_pointer; // just a pointer to find chars in the cmd string like X, Y, Z, E, etc

//static float tt = 0;
//static float bt = 0;

//Inactivity shutdown variables
static unsigned long previous_millis_cmd = 0;
static unsigned long max_inactive_time = 0;
static unsigned long stepper_inactive_time = DEFAULT_STEPPER_DEACTIVE_TIME*1000l;

unsigned long starttime=0;
unsigned long stoptime=0;

static uint8_t tmp_extruder;


bool Stopped=false;
bool force_homing=true;		// When true the printer is stopped till G28 command is executed

#if NUM_SERVOS > 0
Servo servos[NUM_SERVOS];
#endif

bool CooldownNoWait = true;
bool target_direction;

//===========================================================================
//=============================ROUTINES=============================
//===========================================================================

void get_arc_coordinates();
bool setTargetedHotend(int code);
void prepare_move(bool rapid, raster_t* raster_b);
void prepare_raster_move(raster_t* raster_b);

void serial_echopair_P(const char *s_P, float v)
{ serialprintPGM(s_P); SERIAL_ECHO(v); }
void serial_echopair_P(const char *s_P, double v)
{ serialprintPGM(s_P); SERIAL_ECHO(v); }
void serial_echopair_P(const char *s_P, unsigned long v)
{ serialprintPGM(s_P); SERIAL_ECHO(v); }
void serial_echopair_P(const char *s_P, int v)
{ serialprintPGM(s_P); SERIAL_ECHO(v); }
void serial_echopair_P(const char *s_P, short v)
{ serialprintPGM(s_P); SERIAL_ECHO(v); }

extern "C"{
	extern unsigned int __bss_end;
	extern unsigned int __heap_start;
	extern void *__brkval;

	int freeMemory() {
		int free_memory;

		if((int)__brkval == 0)
			free_memory = ((int)&free_memory) - ((int)&__bss_end);
		else
			free_memory = ((int)&free_memory) - ((int)__brkval);

		return free_memory;
	}
}

//adds an command to the main command buffer
//thats really done in a non-safe way.
//needs overworking someday
void enquecommand(const char *cmd)
{
	if(buflen < BUFSIZE)
	{
		//this is dangerous if a mixing of serial and this happsens
		strcpy(&(cmdbuffer[bufindw][0]),cmd);
		SERIAL_ECHO_START;
		SERIAL_ECHOPGM("enqueing \"");
		SERIAL_ECHO(cmdbuffer[bufindw]);
		SERIAL_ECHOLNPGM("\"");
		bufindw= (bufindw + 1)%BUFSIZE;
		buflen += 1;
	}
}

void enquecommand_P(const char *cmd)
{
	if(buflen < BUFSIZE)
	{
		//this is dangerous if a mixing of serial and this happsens
		strcpy_P(&(cmdbuffer[bufindw][0]),cmd);
		SERIAL_ECHO_START;
		SERIAL_ECHOPGM("enqueing \"");
		SERIAL_ECHO(cmdbuffer[bufindw]);
		SERIAL_ECHOLNPGM("\"");
		bufindw= (bufindw + 1)%BUFSIZE;
		buflen += 1;
	}
}

void setup_killpin()
{
#if defined(KILL_PIN) && KILL_PIN > -1
	pinMode(KILL_PIN,INPUT);
	WRITE(KILL_PIN,HIGH);
#endif
}

void setup_photpin()
{
#if defined(PHOTOGRAPH_PIN) && PHOTOGRAPH_PIN > -1
	SET_OUTPUT(PHOTOGRAPH_PIN);
	WRITE(PHOTOGRAPH_PIN, LOW);
#endif
}

void setup_powerhold()
{
#if defined(SUICIDE_PIN) && SUICIDE_PIN > -1
	SET_OUTPUT(SUICIDE_PIN);
	WRITE(SUICIDE_PIN, HIGH);
#endif
#if defined(PS_ON_PIN) && PS_ON_PIN > -1
	SET_OUTPUT(PS_ON_PIN);
#if defined(PS_DEFAULT_OFF)
	WRITE(PS_ON_PIN, PS_ON_ASLEEP);
#else
	WRITE(PS_ON_PIN, PS_ON_AWAKE);
#endif
#endif
}

void suicide()
{
#if defined(SUICIDE_PIN) && SUICIDE_PIN > -1
	SET_OUTPUT(SUICIDE_PIN);
	WRITE(SUICIDE_PIN, LOW);
#endif
}

void extruder_init(){
	 float tmp[]=DEFAULT_AXIS_STEPS_PER_UNIT;
	 axis_steps_per_unit[E_AXIS] = tmp[E_AXIS];
	 tp_init();
}

void dispenser_init(){
	 axis_steps_per_unit[E_AXIS] = DEFAULT_PASTE_DISPENSER_STEPS_PER_UNIT;
}
////////// TwinTeeth Firmware ///////////

void init_tool(short tool){

	Stopped = false;

	switch(tool){
	case PEN_TOOL:
		break;
	case DRILL_TOOL:
		break;
	case LASER_TOOL:
		laser_init();    // Init laser system
		break;
	case EXTRUDER_TOOL:
		extruder_init();
		break;
	case DISPENSER_TOOL:
		dispenser_init();
		break;
	default:
		break;
	}
}

void end_tool(short tool){
	switch(tool){
	case PEN_TOOL:
		break;
	case DRILL_TOOL:
		break;
	case LASER_TOOL:
		laser_end();	// End the laser system
		break;
	case EXTRUDER_TOOL:
		disable_heater();
		tp_end();
		break;
	case DISPENSER_TOOL:
		break;
	default:
		break;
	}
}

void set_active_tool(short tool){
	// Deactivate current tool
	end_tool(active_tool);
	active_tool = tool;
	// Activate new tool
	init_tool(active_tool);
	// Print
	SERIAL_ECHOPAIR("Tool #",active_tool);
	SERIAL_ECHOLNPGM(" activated");
	SERIAL_ECHOLN("");
}


void setup()
{
	setup_killpin();
	setup_powerhold();
	MYSERIAL.begin(BAUDRATE);
	SERIAL_PROTOCOLLNPGM(MSG_START);

	// Check startup - does nothing if bootloader sets MCUSR to 0
	byte mcu = MCUSR;
	if(mcu & 1) SERIAL_ECHOLNPGM(MSG_POWERUP);
	if(mcu & 2) SERIAL_ECHOLNPGM(MSG_EXTERNAL_RESET);
	if(mcu & 4) SERIAL_ECHOLNPGM(MSG_BROWNOUT_RESET);
	if(mcu & 8) SERIAL_ECHOLNPGM(MSG_WATCHDOG_RESET);
	if(mcu & 32) SERIAL_ECHOLNPGM(MSG_SOFTWARE_RESET);
	MCUSR=0;

	SERIAL_ECHOPGM(MSG_DIYOUPCB);
	SERIAL_ECHOLNPGM(VERSION_STRING);
#ifdef STRING_VERSION_CONFIG_H
#ifdef STRING_CONFIG_H_AUTHOR
	SERIAL_ECHOPGM(MSG_CONFIGURATION_VER);
	SERIAL_ECHOPGM(STRING_VERSION_CONFIG_H);
	SERIAL_ECHOPGM(MSG_AUTHOR);
	SERIAL_ECHOLNPGM(STRING_CONFIG_H_AUTHOR);
	SERIAL_ECHOPGM("Compiled: ");
	SERIAL_ECHOLNPGM(__DATE__);
#endif
#endif
	SERIAL_ECHOPGM(MSG_FREE_MEMORY);
	SERIAL_ECHO(freeMemory());
	SERIAL_ECHOPGM(MSG_PLANNER_BUFFER_BYTES);
	SERIAL_ECHOLN((int)sizeof(block_t)*BLOCK_BUFFER_SIZE);
	// loads data from EEPROM if available else uses defaults (and resets step acceleration rate)
	retrieve_tool_config();
	Config_RetrieveSettings();
	set_active_tool(DEFAULT_TOOL);
	print_tool_config();

	plan_init();  // Initialize planner;
	st_init();    // Initialize stepper, this enables interrupts!
	setup_photpin();

	delay_ms(1000);	// wait 1sec to display the splash screen

#if defined(CONTROLLERFAN_PIN) && CONTROLLERFAN_PIN > -1
	SET_OUTPUT(CONTROLLERFAN_PIN); //Set pin used for driver cooling fan
#endif

}

void loop()
{

	if(buflen < (BUFSIZE-1))
		get_command();

	if(buflen)
	{
		process_commands();
		if (!wait_flag){
			buflen = (buflen-1);
			bufindr = (bufindr + 1)%BUFSIZE;
			ClearToSend();
		}
	}
	//check heater every n milliseconds
	if (active_tool == EXTRUDER_TOOL){
		manage_heater();
	}
	manage_inactivity();
	checkHitEndstops();
}
// Check printer status to see if stopped of force homing
//
void check_printer_status(){
	if((strchr(cmdbuffer[bufindw], 'G') != NULL)){
		strchr_pointer = strchr(cmdbuffer[bufindw], 'G');
		switch((int)((strtod(&cmdbuffer[bufindw][strchr_pointer - cmdbuffer[bufindw] + 1], NULL)))){
		case 0:
		case 1:
		case 2:
		case 3:
		case 7:
		case 30:
		case 31:
		case 32:
		case 81:
			if(Stopped == false) { // If printer is stopped by an error the G[0-7] codes are ignored.
				if (force_homing){
					SERIAL_ERROR_START;
					SERIAL_ERRORLNPGM(MSG_FORCE_HOMING);
				}
			}
			else {
				SERIAL_ERROR_START;
				SERIAL_ERRORLNPGM(MSG_ERR_STOPPED);
			}
			SERIAL_PROTOCOLLNPGM(MSG_OK);
			break;
		default:
			break;
		}
	}
}
// Process one line of incoming serial data. Performs an initial filtering
// by removing spaces and comments and capitalizing all letters.
void get_command()
{
	char c;

	while(MYSERIAL.available() > 0 && buflen < BUFSIZE) {
		c = MYSERIAL.read();
		if ((c == '\n') || (c == '\r')) { // End of line reached
			if (serial_count > 0) {  // We have a line
				if(comment_mode != 1){
					comment_mode = NULL;	
					cmdbuffer[bufindw][serial_count] = 0; 
					check_printer_status();
					bufindw = (bufindw + 1)%BUFSIZE;
					buflen += 1;
				}
				serial_count = 0; //clear buffer
			} else { 
				// Empty or comment line. Skip block.
				if (comment_mode == 2){
					comment_mode = NULL; //for new command
				}
				SERIAL_PROTOCOLLNPGM(MSG_OK); // Send CTS message for syncing purposes.
			}    
		} else {
			if (comment_mode != NULL) {
				// Throw away all comment characters
				if (c == ')') {
					// End of comment. Resume line.
					comment_mode = NULL;
				}
			} else {
				if (c < ' ') { 
					// Throw away whitepace and control characters
				} else if (c == '(' && comment_mode != 2 ){
					// Enable comments flag and ignore all characters until ')'.
					comment_mode = 1;
				} else if (c == ';') {
					// Enable comments flag and ignore all characters until EOL.
					comment_mode = 2;
				} else if (serial_count >= (MAX_CMD_SIZE - 1)) {
					// Report line buffer overflow and kill because
					// result can be unpredictable
					SERIAL_PROTOCOLLNPGM(MSG_OVERFLOW);
					kill();
				} else cmdbuffer[bufindw][serial_count++] = c;
			}
		}
	}
}

float code_value()
{
	return (strtod(&cmdbuffer[bufindr][strchr_pointer - cmdbuffer[bufindr] + 1], NULL));
}

long code_value_long()
{
	return (strtol(&cmdbuffer[bufindr][strchr_pointer - cmdbuffer[bufindr] + 1], NULL, 10));
}

bool code_seen(char code)
{
	strchr_pointer = strchr(cmdbuffer[bufindr], code);
	return (strchr_pointer != NULL);  //Return True if a character was found
}

#define DEFINE_PGM_READ_ANY(type, reader)       \
	static inline type pgm_read_any(const type *p)  \
{ return pgm_read_##reader##_near(p); }

DEFINE_PGM_READ_ANY(float,       float);
DEFINE_PGM_READ_ANY(signed char, byte);

#define XYZ_CONSTS_FROM_CONFIG(type, array, CONFIG) \
	static const PROGMEM type array##_P[3] =        \
{ X_##CONFIG, Y_##CONFIG, Z_##CONFIG };     \
	static inline type array(int axis)          \
{ return pgm_read_any(&array##_P[axis]); }

XYZ_CONSTS_FROM_CONFIG(float, base_min_pos,    MIN_POS);
XYZ_CONSTS_FROM_CONFIG(float, base_max_pos,    MAX_POS);
XYZ_CONSTS_FROM_CONFIG(float, base_home_pos,   HOME_POS);
XYZ_CONSTS_FROM_CONFIG(float, max_length,      MAX_LENGTH);
XYZ_CONSTS_FROM_CONFIG(float, home_retract_mm, HOME_RETRACT_MM);
XYZ_CONSTS_FROM_CONFIG(signed char, home_dir,  HOME_DIR);


static void axis_is_at_home(int axis) {

	if (axis != Z_AXIS){
		machine_position[axis] = base_home_pos(axis) + add_homeing[axis];
	    current_position[axis] = machine_position[axis] - tool_config[active_tool].tool_offset[axis];
		min_pos[axis] = base_min_pos(axis) + add_homeing[axis] - tool_config[active_tool].tool_offset[axis];
		max_pos[axis] = base_max_pos(axis) + add_homeing[axis] - tool_config[active_tool].tool_offset[axis];
	}
	else { // Z configuration
		float z_home_pos = (base_home_pos(axis) - z_min_pos[active_tool]) + add_homeing[axis];
		machine_position[axis] = z_home_pos;
		current_position[axis] = machine_position[axis] - tool_config[active_tool].tool_offset[axis];
		max_pos[axis] = current_position[axis];
	}
	current_direction[axis] = 1;
}

#ifdef ENABLE_AUTO_BED_LEVELING

static void run_z_probe(bool adjust) {

	enable_endstops(true);

	float start_z = current_position[Z_AXIS];
	long start_steps = st_get_position(Z_AXIS);

	feedrate = homing_feedrate[Z_AXIS]/4;
	destination[Z_AXIS] = -10;
	prepare_move_raw(adjust);
	st_synchronize();
	endstops_hit_on_purpose();

	enable_endstops(false);
	long stop_steps = st_get_position(Z_AXIS);

	float mm = start_z - float(start_steps - stop_steps) / axis_steps_per_unit[Z_AXIS];
	current_position[Z_AXIS] = mm;
	calculate_delta(current_position);
	if (adjust)
		adjust_delta(destination);
	plan_set_position(delta[X_AXIS], delta[Y_AXIS], delta[Z_AXIS], current_position[E_AXIS]);

}

void do_blocking_move_to(float x, float y, float z, bool adjust) {
	float oldFeedRate = feedrate;

	feedrate = XY_TRAVEL_SPEED;

	destination[X_AXIS] = x;
	destination[Y_AXIS] = y;
	destination[Z_AXIS] = z;

	prepare_move_raw(adjust);

	st_synchronize();

	feedrate = oldFeedRate;
}

static void do_blocking_move_relative(float offset_x, float offset_y, float offset_z, bool adjust) {
	do_blocking_move_to(current_position[X_AXIS] + offset_x, current_position[Y_AXIS] + offset_y, current_position[Z_AXIS] + offset_z, adjust);
}

static void setup_for_endstop_move() {
	saved_feedrate = feedrate;
	saved_feedmultiply = feedmultiply;
	feedmultiply = 100;
	previous_millis_cmd = millis();

}

static void clean_up_after_endstop_move() {
#ifdef ENDSTOPS_ONLY_FOR_HOMING
	enable_endstops(false);
#endif

	feedrate = saved_feedrate;
	feedmultiply = saved_feedmultiply;
	previous_millis_cmd = millis();
}


/// Probe bed height at position (x,y), returns the measured z value
static float probe_pt(float x, float y, float z_before, bool adjust) {
	// move to right place
	// First time looking for platform
	if (z_before == Z_RAISE_BEFORE_PROBING){
		do_blocking_move_to(current_position[X_AXIS], current_position[Y_AXIS], 0, adjust);
	}
	do_blocking_move_to(current_position[X_AXIS], current_position[Y_AXIS], z_before, adjust);
	do_blocking_move_to(x - X_PROBE_OFFSET_FROM_EXTRUDER, y - Y_PROBE_OFFSET_FROM_EXTRUDER, current_position[Z_AXIS],adjust);

	run_z_probe(adjust);
	float measured_z = current_position[Z_AXIS];

	SERIAL_PROTOCOLPGM(MSG_BED);
	SERIAL_PROTOCOLPGM(" x: ");
	SERIAL_PROTOCOL(x);
	SERIAL_PROTOCOLPGM(" y: ");
	SERIAL_PROTOCOL(y);
	SERIAL_PROTOCOLPGM(" z: ");
	SERIAL_PROTOCOL(measured_z);
	SERIAL_PROTOCOLPGM("\n");
	return measured_z;
}

// Print calibration results for plotting or manual frame adjustment.
void print_bed_level(float bed[ACCURATE_BED_LEVELING_POINTS][ACCURATE_BED_LEVELING_POINTS]) {
	for (int y = 0; y < ACCURATE_BED_LEVELING_POINTS; y++) {
		for (int x = 0; x < ACCURATE_BED_LEVELING_POINTS; x++) {
			SERIAL_PROTOCOL_F(bed[x][y], 2);
			SERIAL_PROTOCOLPGM(" ");
		}
		SERIAL_ECHOLN("");
	}
}

// Reset calibration results to zero.
void reset_bed_level() {
	for (int y = 0; y < ACCURATE_BED_LEVELING_POINTS; y++) {
		for (int x = 0; x < ACCURATE_BED_LEVELING_POINTS; x++) {
			bed_level[x][y] = 0.0;
		}
	}
	isBed_leveled = false;
}
#endif // #ifdef ENABLE_AUTO_BED_LEVELING

static void homeaxis(int axis) {
#define HOMEAXIS_DO(LETTER) \
	((LETTER##_MIN_PIN > -1 && LETTER##_HOME_DIR==-1) || (LETTER##_MAX_PIN > -1 && LETTER##_HOME_DIR==1))

	if (axis==X_AXIS ? HOMEAXIS_DO(X) :
		axis==Y_AXIS ? HOMEAXIS_DO(Y) :
		axis==Z_AXIS ? HOMEAXIS_DO(Z) :
		0) {
			int axis_home_dir = home_dir(axis);
			current_position[axis] = 0;
			plan_set_position(current_position[X_AXIS], current_position[Y_AXIS], current_position[Z_AXIS], current_position[E_AXIS]);

				current_position[axis] = 0;
				plan_set_position(current_position[X_AXIS], current_position[Y_AXIS], current_position[Z_AXIS], current_position[E_AXIS]);
				destination[axis] = 1.5 * max_length(axis) * axis_home_dir;
				feedrate = homing_feedrate[axis];
				plan_buffer_line(destination[X_AXIS], destination[Y_AXIS], destination[Z_AXIS], destination[E_AXIS], feedrate/60, active_extruder,NULL);
				st_synchronize();

				enable_endstops(false);  // Ignore Z probe while moving away from the top microswitch.
				current_position[axis] = 0;
				plan_set_position(current_position[X_AXIS], current_position[Y_AXIS], current_position[Z_AXIS], current_position[E_AXIS]);
				destination[axis] = -home_retract_mm(axis) * axis_home_dir;
				plan_buffer_line(destination[X_AXIS], destination[Y_AXIS], destination[Z_AXIS], destination[E_AXIS], feedrate/60, active_extruder,NULL);
				st_synchronize();
				enable_endstops(true);  // Stop ignoring Z probe while moving up to the top microswitch again.

				destination[axis] = 2*home_retract_mm(axis) * axis_home_dir;
#ifdef DELTA
				feedrate = homing_feedrate[axis]/4;
#else
				feedrate = homing_feedrate[axis]/2 ;
#endif
				plan_buffer_line(destination[X_AXIS], destination[Y_AXIS], destination[Z_AXIS], destination[E_AXIS], feedrate/60, active_extruder,NULL);
				st_synchronize();
#ifdef DELTA
				// retrace by the amount specified in endstop_adj
				if (endstop_adj[axis] * axis_home_dir < 0) {
					plan_set_position(current_position[X_AXIS], current_position[Y_AXIS], current_position[Z_AXIS], current_position[E_AXIS]);
					destination[axis] = endstop_adj[axis];
					plan_buffer_line(destination[X_AXIS], destination[Y_AXIS], destination[Z_AXIS], destination[E_AXIS], feedrate/60, active_extruder,NULL);
					st_synchronize();
				}
#endif
				axis_is_at_home(axis);
				destination[axis] = current_position[axis];
				feedrate = 0.0;
				endstops_hit_on_purpose();
				axis_known_position[axis] = true;

				// Retract Servo endstop if enabled
#ifdef SERVO_ENDSTOPS
				if (servo_endstops[axis] > -1) {
					servos[servo_endstops[axis]].write(servo_endstop_angles[axis * 2 + 1]);
				}
#endif
#if defined (ENABLE_AUTO_BED_LEVELING) && (PROBE_SERVO_DEACTIVATION_DELAY > 0)
				if (axis==Z_AXIS) retract_z_probe();
#endif

	}
}
#define HOMEAXIS(LETTER) homeaxis(LETTER##_AXIS)
void refresh_cmd_timeout(void)
{
	previous_millis_cmd = millis();
}

// Home all axis

void home_all(void)
{
	if (active_tool == LASER_TOOL){
		turn_laser_off();		// By default laser is OFF
		focus_status = FOCUS_ST_RESET;
	}

	saved_feedrate = feedrate;
	saved_feedmultiply = feedmultiply;
	feedmultiply = 100;
	previous_millis_cmd = millis();

	enable_endstops(true);

	for(int8_t i=0; i < NUM_AXIS; i++) {
		destination[i] = current_position[i];
	}
	feedrate = 0.0;


	// A delta can only safely home all axis at the same time
	// all axis have to home at the same time

	// Move all carriages up together until the first endstop is hit.
	current_position[X_AXIS] = 0;
	current_position[Y_AXIS] = 0;
	current_position[Z_AXIS] = 0;
	plan_set_position(current_position[X_AXIS], current_position[Y_AXIS], current_position[Z_AXIS], current_position[E_AXIS]);

	destination[X_AXIS] = 3 * Z_MAX_LENGTH;
	destination[Y_AXIS] = 3 * Z_MAX_LENGTH;
	destination[Z_AXIS] = 3 * Z_MAX_LENGTH;
	feedrate = 1.732 * homing_feedrate[X_AXIS];
	plan_buffer_line(destination[X_AXIS], destination[Y_AXIS], destination[Z_AXIS], destination[E_AXIS], feedrate/60, active_extruder,NULL);
	st_synchronize();
	endstops_hit_on_purpose();

	current_position[X_AXIS] = destination[X_AXIS];
	current_position[Y_AXIS] = destination[Y_AXIS];
	current_position[Z_AXIS] = destination[Z_AXIS];

	// take care of back off and rehome now we are all at the top
	HOMEAXIS(X);
	HOMEAXIS(Y);
	HOMEAXIS(Z);

	calculate_delta(current_position);

	plan_set_position(delta[X_AXIS], delta[Y_AXIS], delta[Z_AXIS], current_position[E_AXIS]);

#ifdef ENDSTOPS_ONLY_FOR_HOMING
	enable_endstops(false);
#endif

	feedrate = saved_feedrate;
	feedmultiply = saved_feedmultiply;
	previous_millis_cmd = millis();
	endstops_hit_on_purpose();
	force_homing = false;

}



void process_commands()
{
	unsigned long codenum; //throw away variable

	char *starpos = NULL;
#ifndef NONLINEAR_BED_LEVELING
	float x_tmp, y_tmp, z_tmp, real_z;
#endif
	if(code_seen('G'))
	{
		switch((int)code_value())
		{
		case 0: // G0 Rapid move for CNC
			if(Stopped == false && force_homing == false) {
				get_coordinates(); 
				prepare_move(true,NULL);
				return;
			}
		case 1: // G1
			if(Stopped == false && force_homing == false) {
				get_coordinates(); 
				prepare_move(false,NULL);
				return;
			}
			//	break;
		case 2: // G2  - CW ARC
			if(Stopped == false && force_homing == false) {
				get_arc_coordinates();
				prepare_arc_move(true);
				return;
			}
		case 3: // G3  - CCW ARC
			if(Stopped == false && force_homing == false) {
				get_arc_coordinates();
				prepare_arc_move(false);
				return;
			}
		case 4: // G4 dwell
			codenum = 0;
			if(code_seen('P')) codenum = code_value(); // milliseconds to wait
			if(code_seen('S')) codenum = code_value() * 1000; // seconds to wait

			st_synchronize();
			codenum += millis();  // keep track of when we started waiting
			previous_millis_cmd = millis();
			while(millis()  < codenum ){
				if (active_tool == EXTRUDER_TOOL){
					manage_heater();
				}
				manage_inactivity();
			}
			break;
		case 7: //G7 Execute raster line
			if (active_tool == LASER_TOOL){
				if (focus_status != FOCUS_ST_DONE){
					SERIAL_ECHOLNPGM("Error: laser not focused. Please, focus laser before print PCB");
					return;
				}

				if(Stopped == false && force_homing == false) {
					int raster_raw_length, raster_num_bytes;
					
					raster_t* raster_b = get_new_raster_block();

					if (raster_b == (raster_t*)NULL) {
						wait_flag = true;
						return;
					}

					// We have a valid buffer

					raster_b->busy = true;
					wait_flag = false;
		
					// 
					// Check buffer
					if(code_seen('F')) {
						next_feedrate = code_value();
						if(next_feedrate > 0.0) feedrate = next_feedrate;
					} else {
						next_feedrate = 0;	// Means no feed-rate specified
					}

					raster_b->on_power = raster_b->nominal_power = 0;
					raster_b->off_power = LASER_RASTER_ZERO_LEVEL;

					if(code_seen('P')) {
						int power = (int)code_value();
						if (get_laser_diode() == LASER_RED && power > MAX_LASER_RED_PWR)
							power = MAX_LASER_RED_PWR;
						raster_b->on_power = raster_b->nominal_power = power;
					}

					if (code_seen('L')) raster_raw_length = int(code_value());

					if (code_seen('N')) {
						bool prev_raster_direction = raster_direction;
						raster_direction = (bool)code_value();
						if (prev_raster_direction != raster_direction){
							destination[Y_AXIS] = current_position[Y_AXIS] - (MM_PER_PIXEL * LASER_RASTER_ASPECT_RATIO); // Decrement Y axis
							clamp_to_software_endstops(destination);
							prepare_move(false,NULL);
							buffer_pixels = draw_pixels[0];
						}
					}
					// Get data
					if (code_seen('D')){ 
						raster_num_bytes = base64_decode(raster_b->data, &cmdbuffer[bufindr][strchr_pointer - cmdbuffer[bufindr] + 1], raster_raw_length);
						// Calc pixels. Substract drawing width to num_bytes
						// to see if there are filling bits and remove them
						// If result > 0 means there are more buffers comming, so
						// num pixels are num_bytes * 8.
						raster_b->pixels_per_buffer = raster_num_bytes*8;
						buffer_pixels = buffer_pixels - raster_b->pixels_per_buffer;
						if (buffer_pixels < 0)
							raster_b->pixels_per_buffer += buffer_pixels;
						raster_b->mm_per_pixel = MM_PER_PIXEL;
						raster_b->rpixel_per_mm = round(PIXELS_PER_MM);   // Used to focus
						raster_b->direction = raster_direction;

						// Set direction and calc focus data
						if (!raster_direction) {
							destination[X_AXIS] = current_position[X_AXIS] - (raster_b->mm_per_pixel * raster_b->pixels_per_buffer);
							for (int x = 0; x < fabs(destination[X_AXIS] - current_position[X_AXIS]); x++){
								raster_b->focus[x] = interpolate(current_position[X_AXIS] - x, current_position[Y_AXIS]);
							}
						} else {
							destination[X_AXIS] = current_position[X_AXIS] + (raster_b->mm_per_pixel * raster_b->pixels_per_buffer);
							for (int x = 0; x < fabs(destination[X_AXIS] - current_position[X_AXIS]); x++){
								raster_b->focus[x] = interpolate(current_position[X_AXIS] + x, current_position[Y_AXIS]);
							}
						}
						// Prepare de motion
						prepare_move(false,raster_b);
					}
					//return;
				} // Not stopped
			} // Laser tool
			break;
		case 28: //G28 Home all Axis one at a time
			home_all();
			break;
		case 29: // G29 Detailed Z-Probe, probes the bed at 3 or more points.
			{
				if (active_tool != DRILL_TOOL && active_tool != PEN_TOOL){
					SERIAL_ECHOLNPGM("G29 bed autoleveling command only available on pen or drill toolhead.");
					break;
				}

				// First home all axis
				if(Stopped == false ) {
					home_all();

					bool test = false;
					float bed_test[ACCURATE_BED_LEVELING_POINTS][ACCURATE_BED_LEVELING_POINTS];

					if (code_seen('T')) test = true;

					st_synchronize();

					if (!test) reset_bed_level();

					setup_for_endstop_move();

					feedrate = homing_feedrate[Z_AXIS];

					float z_offset = Z_PROBE_OFFSET_FROM_EXTRUDER;
					if (code_seen(axis_codes[Z_AXIS])) {
						z_offset += code_value();
					}

					for (int yCount=0; yCount < ACCURATE_BED_LEVELING_POINTS; yCount++)
					{
						float yProbe = FRONT_PROBE_BED_POSITION + ACCURATE_BED_LEVELING_GRID_Y * yCount;
						int xStart, xStop, xInc;
						if (yCount % 2) {
							xStart = 0; 
							xStop = ACCURATE_BED_LEVELING_POINTS;
							xInc = 1;
						} else {
							xStart = ACCURATE_BED_LEVELING_POINTS - 1;
							xStop = -1;
							xInc = -1;
						}

						for (int xCount=xStart; xCount != xStop; xCount += xInc)
						{
							float xProbe = LEFT_PROBE_BED_POSITION + ACCURATE_BED_LEVELING_GRID_X * xCount;

							float z_before =  yCount == 0 ? Z_RAISE_BEFORE_PROBING : current_position[Z_AXIS] + Z_RAISE_BETWEEN_PROBINGS;
							float measured_z = probe_pt(xProbe, yProbe, z_before, test);
							if (test)
								bed_test[xCount][yCount] = measured_z + z_offset;
							else
								bed_level[xCount][yCount] = measured_z + z_offset;
						}
					}

					clean_up_after_endstop_move();

					if (test){
						print_bed_level(bed_test);
						st_synchronize();
					} else {
						print_bed_level(bed_level);

						st_synchronize();

						// Mark the tool config as leveled
						// and store in EEPROM
						isBed_leveled = true;
						store_tool_config();
						force_homing = true;
					}

					home_all();
				}
			}
			break;

		case 30: // G30 Single Z Probe
			{
				if(Stopped == false && force_homing == false) {
					st_synchronize();
					// TODO: make sure the bed_level_rotation_matrix is identity or the planner will get set incorectly
					setup_for_endstop_move();

					feedrate = homing_feedrate[Z_AXIS];

					run_z_probe(false);
					SERIAL_PROTOCOLPGM(MSG_BED);
					SERIAL_PROTOCOLPGM(" X: ");
					SERIAL_PROTOCOL(current_position[X_AXIS]);
					SERIAL_PROTOCOLPGM(" Y: ");
					SERIAL_PROTOCOL(current_position[Y_AXIS]);
					SERIAL_PROTOCOLPGM(" Z: ");
					SERIAL_PROTOCOL(current_position[Z_AXIS]);
					SERIAL_PROTOCOLPGM("\n");

					clean_up_after_endstop_move();
				}
			}
			break;

		case 31: // G31 Focus laser
			{
				if (active_tool == LASER_TOOL){
					float tempo = 0;
					int tempw = 0;
					int temph = 0;
					if (code_seen('T')) {  //Once Test mode
						// Disable motors to avoid noise
						finishAndDisableSteppers();
						SERIAL_ECHOLNPGM(MSG_FOCUSING);
						//
						// Focus the laser
						//
						uint8_t focus_pos;

						if ((focus_pos = auto_focus()) == FOCUS_ERROR){
							SERIAL_ERRORLNPGM(MSG_FOCUS_ERROR);
						} else {
							SERIAL_ECHOPGM(MSG_FOCUSED);
							SERIAL_ECHOLN((int)focus_pos);
						}
					} else { // Focus mode
						if(Stopped == false && force_homing == false) {
							if (code_seen('O')) tempo=code_value();
							if (code_seen('W')) tempw=int(code_value());
							if (code_seen('H')) temph=int(code_value());

							if (tempw != 0 && temph != 0){
								draw_pixels[0] = tempw;
								draw_pixels[1] = temph;
								draw_dim[0] = draw_pixels[0]/PIXELS_PER_MM;
								draw_dim[1] = draw_pixels[1]/PIXELS_PER_MM;
								//
								// Reset focusing system
								//
								if (focus_status == FOCUS_ST_RESET || focus_status == FOCUS_ST_DONE || focus_offset != tempo){
									init_focus(tempo, draw_dim[0], draw_dim[1]);
									focus_status = FOCUS_ST_RESET;
								}
								//
								// Gets 4 focus points on the fixture
								//
								if (get_focus_points() == FOCUS_ERROR){
									if (++focus_status == FOCUS_MAX_TRIES)
										focus_status = FOCUS_ST_RESET;
									SERIAL_ERRORLNPGM(MSG_FOCUS_ERROR);
									break;
								} 
								// Move to zero position
								destination[X_AXIS] = 0;
								destination[Y_AXIS] = 0;
								destination[Z_AXIS] = LASER_Z_FOCUS_POS;
								next_feedrate = 0;
								prepare_move(true,NULL);
								st_synchronize();
								focus_status = FOCUS_ST_DONE;
								SERIAL_ECHOLNPGM(MSG_FOCUSED);
							} else {
								focus_status = FOCUS_ST_RESET;
								SERIAL_ERRORLNPGM(MSG_FOCUS_ERROR);
							}
						}
					}
				}
			}
			break;
		case 32: // G32 Init raster
			{
				if (active_tool == LASER_TOOL){
					if(Stopped == false && force_homing == false){
						int tempd = 0;
						int tempw = 0;
						int temph = 0;
						if (focus_status == FOCUS_ST_DONE){
							if (code_seen('D')) set_laser_diode((uint8_t)code_value());
							if (code_seen('W')) tempw=int(code_value());
							if (code_seen('H')) temph=int(code_value());

							if (tempw != 0 && temph != 0 &&
								draw_pixels[0] == tempw && draw_pixels[1] == temph){
									// Init rastering variables
									init_raster();
									buffer_pixels = draw_pixels[0];
									raster_direction = 1;				// Raster direction
									// Move printer to init position
									destination[X_AXIS] = -draw_dim[0]/2;
									destination[Y_AXIS] = +draw_dim[1]/2;
									destination[Z_AXIS] = LASER_Z_FOCUS_POS;
									next_feedrate = 0;
									prepare_move(true,NULL);
									st_synchronize();
									SERIAL_ERRORLNPGM(MSG_RASTERING);
									break;
							}
							focus_status = FOCUS_ST_RESET;
						}
						SERIAL_ERRORLNPGM(MSG_RASTER_ERROR);
					}
				}
			}
			break;
		case 54: // G54 Select active tool
			if(code_seen('P')) { 
				codenum = code_value(); // New active tool
				if (codenum >= 0 && codenum < MAX_TOOL_NUMBER){
					set_active_tool((short)codenum);
				} 
			} else {
				set_active_tool(DEFAULT_TOOL);
			}
			force_homing = true;
			break;
		case 90: // G90
			relative_mode = false;
			break;
		case 91: // G91
			relative_mode = true;
			break;
		case 92: // G92 set offset
			for(int8_t i=0; i < NUM_AXIS; i++) {
				if(code_seen(axis_codes[i])) {
					if(i == E_AXIS) {
						current_position[i] = code_value();
						plan_set_e_position(current_position[E_AXIS]);
					}
					else {
						current_position[i] = code_value() + add_homeing[i] - tool_config[active_tool].tool_offset[i];
						calculate_delta(current_position);
						adjust_delta(current_position);
						plan_set_position(delta[X_AXIS], delta[Y_AXIS], delta[Z_AXIS], current_position[E_AXIS]);
					}
				}
			}
			break;
		case 93: // G93 set program zero for the active tool
			// Set the tool_offset as the current machine position to adjust the program zero position
			// Save the values in EEPROM,
			{
				// Update 
				for(int8_t i=0; i < 3; i++){
					tool_config[active_tool].tool_offset[i] = machine_position[i];
					current_position[i] = 0;
				}
				calculate_delta(current_position);
				adjust_delta(current_position);
				plan_set_position(delta[X_AXIS], delta[Y_AXIS], delta[Z_AXIS], current_position[E_AXIS]);

				SERIAL_PROTOCOLPGM(" Program zero set at: X: ");
				SERIAL_PROTOCOL(machine_position[X_AXIS]);
				SERIAL_PROTOCOLPGM(" Y: ");
				SERIAL_PROTOCOL(machine_position[Y_AXIS]);
				SERIAL_PROTOCOLPGM(" Z: ");
				SERIAL_PROTOCOL(machine_position[Z_AXIS]);
				SERIAL_PROTOCOLPGM("\n");
				SERIAL_PROTOCOLPGM("For Tool #");
				SERIAL_PROTOCOL(active_tool);
				SERIAL_PROTOCOLPGM("\n");
				store_tool_config();
				//		force_homing = true;
			}
			break;
		case 94: // G94 reset all configuration data of active tool or tool specified
			if(code_seen('P')) { 
				codenum = code_value(); // tool
				if (codenum >= 0 && codenum < MAX_TOOL_NUMBER){
					reset_tool_config((short)codenum);
				} 
			} else {
				reset_tool_config(active_tool);
			}
			force_homing = true;
			break;
		case 95: // G95 sets the actual Z axis min value
			{
				float temp = 0.0;
				if(code_seen('Z')) { 
					temp = code_value(); 
					// Z min must be zero or less
					if (temp >= Z_MIN_POS && temp <= Z_MAX_POS){
						min_pos[Z_AXIS] = temp;
					}
					SERIAL_PROTOCOLPGM("Z min set to: ");
					SERIAL_PROTOCOL(min_pos[Z_AXIS]);
					SERIAL_PROTOCOLPGM(" (mm)");
					SERIAL_PROTOCOLPGM("\n");
				}
				//force_homing = true;
			}
			break;
		}
	}
	else if(code_seen('M'))
	{
		switch( (int)code_value() )
		{
		case 2:  // M02 End of program
			break;
		case 3:
			{// M03  power S = % of power from 0 to max. Limit red laser power to MAX_LASER_RED_PWR
				uint8_t tmppwr = 0;
				if (active_tool == LASER_TOOL){
					if(code_seen('S')){
						tmppwr = (uint8_t)code_value();
						if (get_laser_diode() == LASER_RED && tmppwr > MAX_LASER_RED_PWR)
							tmppwr = MAX_LASER_RED_PWR;
						set_laser_power(tmppwr);
					} 
				} 
			}
			break;
		case 5:  // M05 set laser and/or focus diode D0 = OFF, D1 = BLUE, D2 = RED, D3=IR,  Fx = focus
			{
				uint8_t dtmp = 0;
				if (active_tool == LASER_TOOL){
					if(code_seen('D')) {
						dtmp = (uint8_t)code_value();
						if (dtmp > 0)
							set_laser_diode(dtmp);
						else
							turn_laser_off();
					}
					if(code_seen('F')) {
						set_focus_diode((uint8_t)code_value());
					}
				} 
			}
			break;
		case 6:  // M06 Turn laser or spindle off
			if (active_tool == LASER_TOOL){
				turn_laser_on();
			} 
			break;
		case 7:  // M07 Test mode
			if (active_tool == DRILL_TOOL){
				if(code_seen('S')){
					if ((uint8_t)code_value() > 0)
						test_mode = true;
					else
						test_mode = false;
				} 
			} 
			break;
		case 17:
			enable_x();
			enable_y();
			enable_z();
			enable_e0();
			enable_e1();
			enable_e2();
			break;
		case 31: //M31 take time since the start of the SD print or an M109 command
			{
				stoptime=millis();
				char time[30];
				unsigned long t=(stoptime-starttime)/1000;
				int sec,min;
				min=t/60;
				sec=t%60;
				sprintf_P(time, PSTR("%i min, %i sec"), min, sec);
				SERIAL_ECHO_START;
				SERIAL_ECHOLN(time);
				autotempShutdown();
			}
			break;
		case 104: // M104
			if(setTargetedHotend(104)){
				break;
			}
			if (code_seen('S')) setTargetHotend(code_value(), tmp_extruder);
			setWatch();
			break;
		case 140: // M140 set bed temp
			if (code_seen('S')) setTargetBed(code_value());
			break;
		case 105 : // M105
			if(setTargetedHotend(105)){
				break;
			}
#if defined(TEMP_0_PIN) && TEMP_0_PIN > -1
			SERIAL_PROTOCOLPGM("ok T:");
			SERIAL_PROTOCOL_F(degHotend(tmp_extruder),1);
			SERIAL_PROTOCOLPGM(" /");
			SERIAL_PROTOCOL_F(degTargetHotend(tmp_extruder),1);
#if defined(TEMP_BED_PIN) && TEMP_BED_PIN > -1
			SERIAL_PROTOCOLPGM(" B:");
			SERIAL_PROTOCOL_F(degBed(),1);
			SERIAL_PROTOCOLPGM(" /");
			SERIAL_PROTOCOL_F(degTargetBed(),1);
#endif //TEMP_BED_PIN
			for (int8_t cur_extruder = 0; cur_extruder < EXTRUDERS; ++cur_extruder) {
				SERIAL_PROTOCOLPGM(" T");
				SERIAL_PROTOCOL(cur_extruder);
				SERIAL_PROTOCOLPGM(":");
				SERIAL_PROTOCOL_F(degHotend(cur_extruder),1);
				SERIAL_PROTOCOLPGM(" /");
				SERIAL_PROTOCOL_F(degTargetHotend(cur_extruder),1);
			}
#else
			SERIAL_ERROR_START;
			SERIAL_ERRORLNPGM(MSG_ERR_NO_THERMISTORS);
#endif

			SERIAL_PROTOCOLPGM(" @:");
#ifdef EXTRUDER_WATTS
			SERIAL_PROTOCOL((EXTRUDER_WATTS * getHeaterPower(tmp_extruder))/127);
			SERIAL_PROTOCOLPGM("W");
#else
			SERIAL_PROTOCOL(getHeaterPower(tmp_extruder));
#endif

			SERIAL_PROTOCOLPGM(" B@:");
#ifdef BED_WATTS
			SERIAL_PROTOCOL((BED_WATTS * getHeaterPower(-1))/127);
			SERIAL_PROTOCOLPGM("W");
#else
			SERIAL_PROTOCOL(getHeaterPower(-1));
#endif

#ifdef SHOW_TEMP_ADC_VALUES
#if defined(TEMP_BED_PIN) && TEMP_BED_PIN > -1
			SERIAL_PROTOCOLPGM("    ADC B:");
			SERIAL_PROTOCOL_F(degBed(),1);
			SERIAL_PROTOCOLPGM("C->");
			SERIAL_PROTOCOL_F(rawBedTemp()/OVERSAMPLENR,0);
#endif
			for (int8_t cur_extruder = 0; cur_extruder < EXTRUDERS; ++cur_extruder) {
				SERIAL_PROTOCOLPGM("  T");
				SERIAL_PROTOCOL(cur_extruder);
				SERIAL_PROTOCOLPGM(":");
				SERIAL_PROTOCOL_F(degHotend(cur_extruder),1);
				SERIAL_PROTOCOLPGM("C->");
				SERIAL_PROTOCOL_F(rawHotendTemp(cur_extruder)/OVERSAMPLENR,0);
			}
#endif

			SERIAL_PROTOCOLLN("");
			return;
			break;
		case 109:
			{// M109 - Wait for extruder heater to reach target.
				if(setTargetedHotend(109)){
					break;
				}
#ifdef AUTOTEMP
				autotemp_enabled=false;
#endif
				if (code_seen('S')) {
					setTargetHotend(code_value(), tmp_extruder);
					CooldownNoWait = true;
				} else if (code_seen('R')) {
					setTargetHotend(code_value(), tmp_extruder);
					CooldownNoWait = false;
				}
#ifdef AUTOTEMP
				if (code_seen('S')) autotemp_min=code_value();
				if (code_seen('B')) autotemp_max=code_value();
				if (code_seen('F'))
				{
					autotemp_factor=code_value();
					autotemp_enabled=true;
				}
#endif

				setWatch();
				codenum = millis();

				/* See if we are heating up or cooling down */
				target_direction = isHeatingHotend(tmp_extruder); // true if heating, false if cooling

#ifdef TEMP_RESIDENCY_TIME
				long residencyStart;
				residencyStart = -1;
				/* continue to loop until we have reached the target temp
				_and_ until TEMP_RESIDENCY_TIME hasn't passed since we reached it */
				while((residencyStart == -1) ||
					(residencyStart >= 0 && (((unsigned int) (millis() - residencyStart)) < (TEMP_RESIDENCY_TIME * 1000UL))) ) {
#else
				while ( target_direction ? (isHeatingHotend(tmp_extruder)) : (isCoolingHotend(tmp_extruder)&&(CooldownNoWait==false)) ) {
#endif //TEMP_RESIDENCY_TIME
					if( (millis() - codenum) > 1000UL )
					{ //Print Temp Reading and remaining time every 1 second while heating up/cooling down
						SERIAL_PROTOCOLPGM("T:");
						SERIAL_PROTOCOL_F(degHotend(tmp_extruder),1);
						SERIAL_PROTOCOLPGM(" E:");
						SERIAL_PROTOCOL((int)tmp_extruder);
#ifdef TEMP_RESIDENCY_TIME
						SERIAL_PROTOCOLPGM(" W:");
						if(residencyStart > -1)
						{
							codenum = ((TEMP_RESIDENCY_TIME * 1000UL) - (millis() - residencyStart)) / 1000UL;
							SERIAL_PROTOCOLLN( codenum );
						}
						else
						{
							SERIAL_PROTOCOLLN( "?" );
						}
#else
						SERIAL_PROTOCOLLN("");
#endif
						codenum = millis();
					}
					if (active_tool == EXTRUDER_TOOL){
						manage_heater();
					}
					manage_inactivity();
#ifdef TEMP_RESIDENCY_TIME
					/* start/restart the TEMP_RESIDENCY_TIME timer whenever we reach target temp for the first time
					or when current temp falls outside the hysteresis after target temp was reached */
					if ((residencyStart == -1 &&  target_direction && (degHotend(tmp_extruder) >= (degTargetHotend(tmp_extruder)-TEMP_WINDOW))) ||
						(residencyStart == -1 && !target_direction && (degHotend(tmp_extruder) <= (degTargetHotend(tmp_extruder)+TEMP_WINDOW))) ||
						(residencyStart > -1 && labs(degHotend(tmp_extruder) - degTargetHotend(tmp_extruder)) > TEMP_HYSTERESIS) )
					{
						residencyStart = millis();
					}
#endif //TEMP_RESIDENCY_TIME
				}
				starttime=millis();
				previous_millis_cmd = millis();
			}
			break;
		case 190: // M190 - Wait for bed heater to reach target.
#if defined(TEMP_BED_PIN) && TEMP_BED_PIN > -1
			if (code_seen('S')) {
				setTargetBed(code_value());
				CooldownNoWait = true;
			} else if (code_seen('R')) {
				setTargetBed(code_value());
				CooldownNoWait = false;
			}
			codenum = millis();

			target_direction = isHeatingBed(); // true if heating, false if cooling

			while ( target_direction ? (isHeatingBed()) : (isCoolingBed()&&(CooldownNoWait==false)) )
			{
				if(( millis() - codenum) > 1000 ) //Print Temp Reading every 1 second while heating up.
				{
					float tt=degHotend(active_extruder);
					SERIAL_PROTOCOLPGM("T:");
					SERIAL_PROTOCOL(tt);
					SERIAL_PROTOCOLPGM(" E:");
					SERIAL_PROTOCOL((int)active_extruder);
					SERIAL_PROTOCOLPGM(" B:");
					SERIAL_PROTOCOL_F(degBed(),1);
					SERIAL_PROTOCOLLN("");
					codenum = millis();
				}
				if (active_tool == EXTRUDER_TOOL){
					manage_heater();
				}
				manage_inactivity();
			}
			previous_millis_cmd = millis();
#endif
			break;

#if defined(FAN_PIN) && FAN_PIN > -1
		case 106: //M106 Fan On
			if (code_seen('S')){
				fanSpeed=constrain(code_value(),0,255);
			}
			else {
				fanSpeed=255;
			}
			break;
		case 107: //M107 Fan Off
			fanSpeed = 0;
			break;
#endif //FAN_PIN
#if defined(PS_ON_PIN) && PS_ON_PIN > -1
		case 80: // M80 - Turn on Power Supply
			SET_OUTPUT(PS_ON_PIN); //GND
			WRITE(PS_ON_PIN, PS_ON_AWAKE);

			// If you have a switch on suicide pin, this is useful
			// if you want to start another print with suicide feature after
			// a print without suicide...
#if defined SUICIDE_PIN && SUICIDE_PIN > -1
			SET_OUTPUT(SUICIDE_PIN);
			WRITE(SUICIDE_PIN, HIGH);
#endif
			break;
#endif

		case 81: // M81 - Turn off Power Supply
			disable_heater();
			st_synchronize();
			disable_e0();
			disable_e1();
			disable_e2();
			finishAndDisableSteppers();
			fanSpeed = 0;
			delay(1000); // Wait a little before to switch off
#if defined(SUICIDE_PIN) && SUICIDE_PIN > -1
			st_synchronize();
			suicide();
#elif defined(PS_ON_PIN) && PS_ON_PIN > -1
			SET_OUTPUT(PS_ON_PIN);
			WRITE(PS_ON_PIN, PS_ON_ASLEEP);
#endif
			break;
		case 82:
			axis_relative_modes[3] = false;
			break;
		case 83:
			axis_relative_modes[3] = true;
			break;
		case 18: //compatibility
		case 84: // M84
			if(code_seen('S')){
				stepper_inactive_time = code_value() * 1000;
			}
			else
			{
				bool all_axis = !((code_seen(axis_codes[0])) || (code_seen(axis_codes[1])) || (code_seen(axis_codes[2]))|| (code_seen(axis_codes[3])));
				if(all_axis)
				{
					st_synchronize();
					disable_e0();
					disable_e1();
					disable_e2();
					finishAndDisableSteppers();
				}
				else
				{
					st_synchronize();
					if(code_seen('X')) disable_x();
					if(code_seen('Y')) disable_y();
					if(code_seen('Z')) disable_z();
#if ((E0_ENABLE_PIN != X_ENABLE_PIN) && (E1_ENABLE_PIN != Y_ENABLE_PIN)) // Only enable on boards that have seperate ENABLE_PINS
					if(code_seen('E')) {
						disable_e0();
						disable_e1();
						disable_e2();
					}
#endif
				}
			}
			break;
		case 85: // M85
			code_seen('S');
			max_inactive_time = code_value() * 1000;
			break;
		case 92: // M92
			for(int8_t i=0; i < NUM_AXIS; i++)
			{
				if(code_seen(axis_codes[i]))
				{
					if(i == 3) { // E
						float value = code_value();
						if(value < 20.0) {
							float factor = axis_steps_per_unit[i] / value; // increase e constants if M92 E14 is given for netfab.
							max_e_jerk *= factor;
							max_feedrate[i] *= factor;
							axis_steps_per_sqr_second[i] *= factor;
						}
						axis_steps_per_unit[i] = value;
					}
					else {
						axis_steps_per_unit[i] = code_value();
					}
				}
			}
			break;
		case 115: // M115
			SERIAL_PROTOCOLPGM(MSG_M115_REPORT);
			break;
		case 117: // M117 display message
			starpos = (strchr(strchr_pointer + 5,'*'));
			if(starpos!=NULL)
				*(starpos-1)='\0';
			break;
		case 114: // M114
			SERIAL_PROTOCOLPGM("Currrent: X:");
			SERIAL_PROTOCOL(current_position[X_AXIS]);
			SERIAL_PROTOCOLPGM(" Y:");
			SERIAL_PROTOCOL(current_position[Y_AXIS]);
			SERIAL_PROTOCOLPGM(" Z:");
			SERIAL_PROTOCOL(current_position[Z_AXIS]);
			SERIAL_PROTOCOLPGM(" E:");
			SERIAL_PROTOCOL(current_position[E_AXIS]);
			SERIAL_PROTOCOLLN("");
			SERIAL_PROTOCOLPGM("Machine:  X:");
			SERIAL_PROTOCOL(machine_position[X_AXIS]);
			SERIAL_PROTOCOLPGM(" Y:");
			SERIAL_PROTOCOL(machine_position[Y_AXIS]);
			SERIAL_PROTOCOLPGM(" Z:");
			SERIAL_PROTOCOL(machine_position[Z_AXIS]);
			SERIAL_PROTOCOLLN("");
			SERIAL_PROTOCOLPGM("Z Min:");
			SERIAL_PROTOCOL(min_pos[Z_AXIS]);
			SERIAL_PROTOCOLPGM(" Max:");
			SERIAL_PROTOCOL(max_pos[Z_AXIS]);
			SERIAL_PROTOCOLLN("");
			break;
		case 120: // M120
			enable_endstops(false) ;
			break;
		case 121: // M121
			enable_endstops(true) ;
			break;
		case 119: // M119
			SERIAL_PROTOCOLLN(MSG_M119_REPORT);
#if defined(X_MIN_PIN) && X_MIN_PIN > -1
			SERIAL_PROTOCOLPGM(MSG_X_MIN);
			SERIAL_PROTOCOLLN(((READ(X_MIN_PIN)^X_MIN_ENDSTOP_INVERTING)?MSG_ENDSTOP_HIT:MSG_ENDSTOP_OPEN));
#endif
#if defined(X_MAX_PIN) && X_MAX_PIN > -1
			SERIAL_PROTOCOLPGM(MSG_X_MAX);
			SERIAL_PROTOCOLLN(((READ(X_MAX_PIN)^X_MAX_ENDSTOP_INVERTING)?MSG_ENDSTOP_HIT:MSG_ENDSTOP_OPEN));
#endif
#if defined(Y_MIN_PIN) && Y_MIN_PIN > -1
			SERIAL_PROTOCOLPGM(MSG_Y_MIN);
			SERIAL_PROTOCOLLN(((READ(Y_MIN_PIN)^Y_MIN_ENDSTOP_INVERTING)?MSG_ENDSTOP_HIT:MSG_ENDSTOP_OPEN));
#endif
#if defined(Y_MAX_PIN) && Y_MAX_PIN > -1
			SERIAL_PROTOCOLPGM(MSG_Y_MAX);
			SERIAL_PROTOCOLLN(((READ(Y_MAX_PIN)^Y_MAX_ENDSTOP_INVERTING)?MSG_ENDSTOP_HIT:MSG_ENDSTOP_OPEN));
#endif
#if defined(Z_MIN_PIN) && Z_MIN_PIN > -1
			SERIAL_PROTOCOLPGM(MSG_Z_MIN);
			SERIAL_PROTOCOLLN(((READ(Z_MIN_PIN)^Z_MIN_ENDSTOP_INVERTING)?MSG_ENDSTOP_HIT:MSG_ENDSTOP_OPEN));
#endif
#if defined(Z_MAX_PIN) && Z_MAX_PIN > -1
			SERIAL_PROTOCOLPGM(MSG_Z_MAX);
			SERIAL_PROTOCOLLN(((READ(Z_MAX_PIN)^Z_MAX_ENDSTOP_INVERTING)?MSG_ENDSTOP_HIT:MSG_ENDSTOP_OPEN));
#endif
			break;
			//TODO: update for all axis, use for loop
		case 200: // M200 D<millimeters> set filament diameter and set E axis units to cubic millimeters (use S0 to set back to millimeters).
			{
				float area = .0;
				float radius = .0;
				if(code_seen('D')) {
					radius = (float)code_value() * .5;
					if(radius == 0) {
						area = 1;
					} else {
						area = M_PI * pow(radius, 2);
					}
				} else {
					//reserved for setting filament diameter via UFID or filament measuring device
					break;
				}
				tmp_extruder = active_extruder;
				if(code_seen('T')) {
					tmp_extruder = code_value();
					if(tmp_extruder >= EXTRUDERS) {
						SERIAL_ECHO_START;
						SERIAL_ECHO(MSG_M200_INVALID_EXTRUDER);
					}
					SERIAL_ECHOLN(tmp_extruder);
					break;
				}
				volumetric_multiplier[tmp_extruder] = 1 / area;
			}
			break;
		case 201: // M201
			for(int8_t i=0; i < NUM_AXIS; i++)
			{
				if(code_seen(axis_codes[i]))
				{
					max_acceleration_units_per_sq_second[i] = code_value();
				}
			}
			// steps per sq second need to be updated to agree with the units per sq second (as they are what is used in the planner)
			reset_acceleration_rates();
			break;
		case 203: // M203 max feedrate mm/sec
			for(int8_t i=0; i < NUM_AXIS; i++) {
				if(code_seen(axis_codes[i])) max_feedrate[i] = code_value();
			}
			break;
		case 204: // M204 acclereration S normal moves T filmanent only moves
			{
				if(code_seen('S')) acceleration = code_value() ;
				if(code_seen('T')) retract_acceleration = code_value() ;
			}
			break;
		case 205: //M205 advanced settings:  minimum travel speed S=while printing T=travel only,  B=minimum segment time X= maximum xy jerk, Z=maximum Z jerk
			{
				if(code_seen('S')) minimumfeedrate = code_value();
				if(code_seen('T')) mintravelfeedrate = code_value();
				if(code_seen('B')) minsegmenttime = code_value() ;
				if(code_seen('X')) max_xy_jerk = code_value() ;
				if(code_seen('Z')) max_z_jerk = code_value() ;
				if(code_seen('E')) max_e_jerk = code_value() ;
			}
			break;
		case 206: // M206 additional homeing offset
			for(int8_t i=0; i < 3; i++)
			{
				if(code_seen(axis_codes[i])) add_homeing[i] = code_value();
			}
			break;
#ifdef DELTA
		case 666: // M666 set delta endstop adjustemnt
			for(int8_t i=0; i < 3; i++)
			{
				if(code_seen(axis_codes[i])) endstop_adj[i] = code_value();
			}
			break;
#endif
		case 220: // M220 S<factor in percent>- set speed factor override percentage
			{
				if(code_seen('S'))
				{
					feedmultiply = code_value() ;
				}
			}
			break;
		case 221: // M221 S<factor in percent>- set extrude factor override percentage
			{
				if(code_seen('S'))
				{
					extrudemultiply = code_value() ;
				}
			}
			break;
#ifdef PIDTEMP
		case 301: // M301
			{
				if(code_seen('P')) Kp = code_value();
				if(code_seen('I')) Ki = scalePID_i(code_value());
				if(code_seen('D')) Kd = scalePID_d(code_value());

#ifdef PID_ADD_EXTRUSION_RATE
				if(code_seen('C')) Kc = code_value();
#endif

				updatePID();
				SERIAL_PROTOCOL(MSG_OK);
				SERIAL_PROTOCOL(" p:");
				SERIAL_PROTOCOL(Kp);
				SERIAL_PROTOCOL(" i:");
				SERIAL_PROTOCOL(unscalePID_i(Ki));
				SERIAL_PROTOCOL(" d:");
				SERIAL_PROTOCOL(unscalePID_d(Kd));
#ifdef PID_ADD_EXTRUSION_RATE
				SERIAL_PROTOCOL(" c:");
				//Kc does not have scaling applied above, or in resetting defaults
				SERIAL_PROTOCOL(Kc);
#endif
				SERIAL_PROTOCOLLN("");
			}
			break;
#endif //PIDTEMP
#ifdef PIDTEMPBED
		case 304: // M304
			{
				if(code_seen('P')) bedKp = code_value();
				if(code_seen('I')) bedKi = scalePID_i(code_value());
				if(code_seen('D')) bedKd = scalePID_d(code_value());

				updatePID();
				SERIAL_PROTOCOL(MSG_OK);
				SERIAL_PROTOCOL(" p:");
				SERIAL_PROTOCOL(bedKp);
				SERIAL_PROTOCOL(" i:");
				SERIAL_PROTOCOL(unscalePID_i(bedKi));
				SERIAL_PROTOCOL(" d:");
				SERIAL_PROTOCOL(unscalePID_d(bedKd));
				SERIAL_PROTOCOLLN("");
			}
			break;
#endif //PIDTEMP
		case 240: // M240  Triggers a camera by emulating a Canon RC-1 : http://www.doc-diy.net/photo/rc-1_hacked/
			{
#if defined(PHOTOGRAPH_PIN) && PHOTOGRAPH_PIN > -1
				const uint8_t NUM_PULSES=16;
				const float PULSE_LENGTH=0.01524;
				for(int i=0; i < NUM_PULSES; i++) {
					WRITE(PHOTOGRAPH_PIN, HIGH);
					_delay_ms(PULSE_LENGTH);
					WRITE(PHOTOGRAPH_PIN, LOW);
					_delay_ms(PULSE_LENGTH);
				}
				delay(7.33);
				for(int i=0; i < NUM_PULSES; i++) {
					WRITE(PHOTOGRAPH_PIN, HIGH);
					_delay_ms(PULSE_LENGTH);
					WRITE(PHOTOGRAPH_PIN, LOW);
					_delay_ms(PULSE_LENGTH);
				}
#endif
			}
			break;
#ifdef PREVENT_DANGEROUS_EXTRUDE
		case 302: // allow cold extrudes, or set the minimum extrude temperature
			{
				float temp = .0;
				if (code_seen('S')) temp=code_value();
				set_extrude_min_temp(temp);
			}
			break;
#endif
		case 303: // M303 PID autotune
			{
				float temp = 150.0;
				int e=0;
				int c=5;
				if (code_seen('E')) e=code_value();
				if (e<0)
					temp=70;
				if (code_seen('S')) temp=code_value();
				if (code_seen('C')) c=code_value();
				PID_autotune(temp, e, c);
			}
			break;
		case 400: // M400 finish all moves
			{
				st_synchronize();
			}
			break;

		case 500: // M500 Store settings in EEPROM
			{
				Config_StoreSettings();
			}
			break;
		case 501: // M501 Read settings from EEPROM
			{
				Config_RetrieveSettings();
			}
			break;
		case 502: // M502 Revert to default settings
			{
				Config_ResetDefault();
			}
			break;
		case 503: // M503 print settings currently in memory
			{
				Config_PrintSettings();
			}
			break;
		case 504: // M504 Read tool config from EEPROM
			{
				store_tool_config();
			}
			break;
		case 505: // M505 Retrieve all tool config
			{
				retrieve_tool_config();
			}
			break;
		case 506: // M506 reset tool config data
			{
				reset_all_config();
			}
			break;
		case 507:  // M507 print tool config data
			{
				print_tool_config();
			}
			break;
		case 508:// M508 Set seek rate
			{
				float temp = 0.0;
				if (code_seen('F')) temp=code_value();
				if (temp <= DEFAULT_MAX_SEEK_RATE){
					seek_rate = temp;
				}
			}
			break;

#ifdef ABORT_ON_ENDSTOP_HIT_FEATURE_ENABLED
		case 540:
			{
				if(code_seen('S')) abort_on_endstop_hit = code_value() > 0;
			}
			break;
#endif
		case 350: // M350 Set microstepping mode. Warning: Steps per unit remains unchanged. S code sets stepping mode for all drivers.
			{
#if defined(X_MS1_PIN) && X_MS1_PIN > -1
				if(code_seen('S')) for(int i=0;i<=4;i++) microstep_mode(i,code_value());
				for(int i=0;i<NUM_AXIS;i++) if(code_seen(axis_codes[i])) microstep_mode(i,(uint8_t)code_value());
				if(code_seen('B')) microstep_mode(4,code_value());
				microstep_readings();
#endif
			}
			break;
		case 351: // M351 Toggle MS1 MS2 pins directly, S# determines MS1 or MS2, X# sets the pin high/low.
			{
#if defined(X_MS1_PIN) && X_MS1_PIN > -1
				if(code_seen('S')) switch((int)code_value())
				{
				case 1:
					for(int i=0;i<NUM_AXIS;i++) if(code_seen(axis_codes[i])) microstep_ms(i,code_value(),-1);
					if(code_seen('B')) microstep_ms(4,code_value(),-1);
					break;
				case 2:
					for(int i=0;i<NUM_AXIS;i++) if(code_seen(axis_codes[i])) microstep_ms(i,-1,code_value());
					if(code_seen('B')) microstep_ms(4,-1,code_value());
					break;
				}
				microstep_readings();
#endif
			}
			break;
		case 999: // M999: Restart after being stopped
			Stopped = false;
			FlushSerialRequestResend();
			break;
		}
	}

	else if(code_seen('T'))
	{
		tmp_extruder = code_value();
		if(tmp_extruder >= EXTRUDERS) {
			SERIAL_ECHO_START;
			SERIAL_ECHO("T");
			SERIAL_ECHO(tmp_extruder);
			SERIAL_ECHOLN(MSG_INVALID_EXTRUDER);
		}
		else {
			boolean make_move = false;
			if(code_seen('F')) {
				make_move = true;
				next_feedrate = code_value();
				if(next_feedrate > 0.0) {
					feedrate = next_feedrate;
				}
			}
			SERIAL_ECHO_START;
			SERIAL_ECHO(MSG_ACTIVE_EXTRUDER);
			SERIAL_PROTOCOLLN((int)active_extruder);
		}
	}

	else
	{
		SERIAL_ECHOPGM(MSG_UNKNOWN_COMMAND);
		SERIAL_ECHO(cmdbuffer[bufindr]);
		SERIAL_ECHOLNPGM("\"");
	}

	//ClearToSend();
}

void FlushSerialRequestResend()
{
	MYSERIAL.flush();
	SERIAL_PROTOCOLPGM(MSG_RESEND);
	ClearToSend();
}

void ClearToSend()
{
	previous_millis_cmd = millis();
	SERIAL_PROTOCOLLNPGM(MSG_OK);
}


void get_coordinates()
{
	bool seen[4]={false,false,false,false};
	for(int8_t i=0; i < NUM_AXIS; i++) {
		if(code_seen(axis_codes[i]))
		{
			destination[i] = (float)code_value() + (axis_relative_modes[i] || relative_mode)*current_position[i];
			seen[i]=true;
		}
		else destination[i] = current_position[i]; //Are these else lines really needed?
	}
	
	// Get feed rate
	if(code_seen('F')) {
		next_feedrate = code_value();
		if(next_feedrate > 0.0) feedrate = next_feedrate;
	} else {
		next_feedrate = 0;	// Means no feed-rate specified
	}
}

void get_arc_coordinates()
{
#ifdef SF_ARC_FIX
	bool relative_mode_backup = relative_mode;
	relative_mode = true;
#endif
	get_coordinates();
#ifdef SF_ARC_FIX
	relative_mode=relative_mode_backup;
#endif

	if(code_seen('I')) {
		offset[0] = code_value();
	}
	else {
		offset[0] = 0.0;
	}
	if(code_seen('J')) {
		offset[1] = code_value();
	}
	else {
		offset[1] = 0.0;
	}

	// Get feed rate
	if(code_seen('F')) {
		next_feedrate = code_value();
		if(next_feedrate > 0.0) feedrate = next_feedrate;
	} else {
		next_feedrate = 0;	// Means no feed-rate specified
	}
}

void clamp_to_software_endstops(float target[3])
{
	if (min_software_endstops) {
		if (target[X_AXIS] < min_pos[X_AXIS]) target[X_AXIS] = min_pos[X_AXIS];
		if (target[Y_AXIS] < min_pos[Y_AXIS]) target[Y_AXIS] = min_pos[Y_AXIS];
		if (target[Z_AXIS] < min_pos[Z_AXIS]) target[Z_AXIS] = min_pos[Z_AXIS];
	}

	if (max_software_endstops) {
		if (target[X_AXIS] > max_pos[X_AXIS]) target[X_AXIS] = max_pos[X_AXIS];
		if (target[Y_AXIS] > max_pos[Y_AXIS]) target[Y_AXIS] = max_pos[Y_AXIS];
		if (target[Z_AXIS] > max_pos[Z_AXIS]) target[Z_AXIS] = max_pos[Z_AXIS];
	}
}

#ifdef DELTA
void calculate_delta(float cartesian[3])
{
	//
	// Add offsets. In this way the robot moves always in machine coords
	// and autoleveing system works always even with tool offsets
	//
	register float x  = cartesian[X_AXIS] + tool_config[active_tool].tool_offset[X_AXIS];
	register float y  = cartesian[Y_AXIS] + tool_config[active_tool].tool_offset[Y_AXIS];
	register float z  = cartesian[Z_AXIS] - tool_config[active_tool].tool_offset[Z_AXIS];

	delta[X_AXIS] = sqrt(DELTA_DIAGONAL_ROD_2
		- sq(DELTA_TOWER1_X+x)  // Changed sign for DIYPCB
		- sq(DELTA_TOWER1_Y+y)  // Changed sign for DIYPCB
		) + z;
	delta[Y_AXIS] = sqrt(DELTA_DIAGONAL_ROD_2
		- sq(DELTA_TOWER2_X+x) // Changed sign for DIYPCB
		- sq(DELTA_TOWER2_Y+y) // Changed sign for DIYPCB
		) + z;
	delta[Z_AXIS] = sqrt(DELTA_DIAGONAL_ROD_2
		- sq(DELTA_TOWER3_X+x) // Changed sign for DIYPCB
		- sq(DELTA_TOWER3_Y+y) // Changed sign for DIYPCB
		) + z;
	/*
	SERIAL_ECHOPGM("cartesian x="); SERIAL_ECHO(cartesian[X_AXIS]);
	SERIAL_ECHOPGM(" y="); SERIAL_ECHO(cartesian[Y_AXIS]);
	SERIAL_ECHOPGM(" z="); SERIAL_ECHOLN(cartesian[Z_AXIS]);

	SERIAL_ECHOPGM("delta x="); SERIAL_ECHO(delta[X_AXIS]);
	SERIAL_ECHOPGM(" y="); SERIAL_ECHO(delta[Y_AXIS]);
	SERIAL_ECHOPGM(" z="); SERIAL_ECHOLN(delta[Z_AXIS]);
	*/
}

// Adjust print surface height by linear interpolation over the bed_level array.
void adjust_delta(float cartesian[3])
{

#ifdef BACKLASH_COMPENSATION
	// BACKLASH compensation
	// Check if we have to insert an extra movement in X
	// first we check if the axis has some change. Then if it changed direction and lash is not cero.
	//
	
	if (!isSameDirection(X_AXIS) && BACKLASH_COMPENSATION > 0.0f) {
		current_direction[X_AXIS] = -current_direction[X_AXIS]; // reverse the direction of movement
		delta[X_AXIS] += BACKLASH_COMPENSATION * current_direction[X_AXIS]; // calculate countermeasure position
	}

	// Check if we have to insert an extra movement in Y
	// first we check if the axis has some change. Then if it changed direction and lash is not cero.
	//

	if (!isSameDirection(Y_AXIS) && BACKLASH_COMPENSATION > 0.0f) {
		current_direction[Y_AXIS] = -current_direction[Y_AXIS]; // reverse the direction of movement
		delta[Y_AXIS] += BACKLASH_COMPENSATION * current_direction[Y_AXIS]; // calculate countermeasure position
	}

	// Check if we have to insert an extra movement in Z
	// first we check if the axis has some change. Then if it changed direction and lash is not cero.
	//

	if (!isSameDirection(Z_AXIS) && BACKLASH_COMPENSATION > 0.0f) {
		current_direction[Z_AXIS] = -current_direction[Z_AXIS]; // reverse the direction of movement
		delta[Z_AXIS] += BACKLASH_COMPENSATION * current_direction[Z_AXIS]; // calculate countermeasure position
	}
#endif
	//
	// Add tool offsets
	//

	float x  = cartesian[X_AXIS] + tool_config[active_tool].tool_offset[X_AXIS];
	float y  = cartesian[Y_AXIS] + tool_config[active_tool].tool_offset[Y_AXIS];

	//
	// Now adjust for leveling
	//
	int half = (ACCURATE_BED_LEVELING_POINTS - 1) / 2;

	float grid_x = max(0.001-half, min(half-0.001, (x / ACCURATE_BED_LEVELING_GRID_X)));
	float grid_y = max(0.001-half, min(half-0.001, (y / ACCURATE_BED_LEVELING_GRID_Y)));

	int floor_x = floor(grid_x);
	int floor_y = floor(grid_y);
	float ratio_x = grid_x - floor_x;
	float ratio_y = grid_y - floor_y;
	float z1 = bed_level[floor_x+half][floor_y+half];
	float z2 = bed_level[floor_x+half][floor_y+half+1];
	float z3 = bed_level[floor_x+half+1][floor_y+half];
	float z4 = bed_level[floor_x+half+1][floor_y+half+1];
	float left = (1-ratio_y)*z1 + ratio_y*z2;
	float right = (1-ratio_y)*z3 + ratio_y*z4;
	float offset = (1-ratio_x)*left + ratio_x*right;
	

	delta[X_AXIS] += offset;
	delta[Y_AXIS] += offset;
	delta[Z_AXIS] += offset;

	
	/*
	SERIAL_ECHOPGM("grid_x="); SERIAL_ECHO(grid_x);
	SERIAL_ECHOPGM(" grid_y="); SERIAL_ECHO(grid_y);
	SERIAL_ECHOPGM(" floor_x="); SERIAL_ECHO(floor_x);
	SERIAL_ECHOPGM(" floor_y="); SERIAL_ECHO(floor_y);
	SERIAL_ECHOPGM(" ratio_x="); SERIAL_ECHO(ratio_x);
	SERIAL_ECHOPGM(" ratio_y="); SERIAL_ECHO(ratio_y);
	SERIAL_ECHOPGM(" z1="); SERIAL_ECHO(z1);
	SERIAL_ECHOPGM(" z2="); SERIAL_ECHO(z2);
	SERIAL_ECHOPGM(" z3="); SERIAL_ECHO(z3);
	SERIAL_ECHOPGM(" z4="); SERIAL_ECHO(z4);
	SERIAL_ECHOPGM(" left="); SERIAL_ECHO(left);
	SERIAL_ECHOPGM(" right="); SERIAL_ECHO(right);
	SERIAL_ECHOPGM(" offset="); SERIAL_ECHOLN(offset);
	*/
}

void prepare_move_raw(bool adjust)
{
	previous_millis_cmd = millis();
	calculate_delta(destination);
	if (adjust)
		adjust_delta(destination);

	plan_buffer_line(delta[X_AXIS], delta[Y_AXIS], delta[Z_AXIS],
		destination[E_AXIS], feedrate*feedmultiply/60/100.0,
		active_extruder,NULL);

	for(int8_t i=0; i < NUM_AXIS; i++) {
		current_position[i] = destination[i];
		machine_position[i] = current_position[i] + tool_config[active_tool].tool_offset[i];
	}
}
#endif
/*
** prepare_raster_move() - Prepare raster move
**/
void prepare_raster_move(raster_t* raster_b)
{
	float difference[NUM_AXIS];

	clamp_to_software_endstops(destination);

	previous_millis_cmd = millis();

	for (int8_t i=0; i < NUM_AXIS; i++) {
		difference[i] = destination[i] - current_position[i];
	}

	float cartesian_mm = sqrt(sq(difference[X_AXIS]) +
		sq(difference[Y_AXIS]) +
		sq(difference[Z_AXIS]));

	if (cartesian_mm < 0.000001) { return; }
	
	raster_b->cartesian_blk_mm = cartesian_mm;
	
	calculate_delta(destination);
	adjust_delta(destination);

	plan_buffer_line(delta[X_AXIS], delta[Y_AXIS], delta[Z_AXIS],
			destination[E_AXIS], feedrate*feedmultiply/60/100.0,
			active_extruder,raster_b);

	for (int8_t i=0; i < NUM_AXIS; i++) {
		current_position[i] = destination[i];
		machine_position[i] = current_position[i] + tool_config[active_tool].tool_offset[i];
	}
}
/*
** prepare_move() - Prepare raster & vectorial move
** Rapid  - true means G0 (seek move), else G1 (feed move)
**/
void prepare_move(bool rapid, raster_t* raster_b)
{
	float myfeedrate;

	// Change feed rate to seek_rate if G0

	myfeedrate = feedrate;

	if (rapid && next_feedrate == 0){ // G0
		myfeedrate = seek_rate;
	}

	clamp_to_software_endstops(destination);

	previous_millis_cmd = millis();

	float difference[NUM_AXIS];

	// If test mode negative Z coordinates set to zero

	if (test_mode){
		if (destination[Z_AXIS] < 0)
			destination[Z_AXIS] = 0;
	}

	for (int8_t i=0; i < NUM_AXIS; i++) {
		difference[i] = destination[i] - current_position[i];
	}

	float cartesian_mm = sqrt(sq(difference[X_AXIS]) +
		sq(difference[Y_AXIS]) +
		sq(difference[Z_AXIS]));
	if (cartesian_mm < 0.000001) { cartesian_mm = abs(difference[E_AXIS]); }
	if (cartesian_mm < 0.000001) { return; }

	float seconds = 6000 * cartesian_mm / myfeedrate / feedmultiply;
	int steps = max(1, int(DELTA_SEGMENTS_PER_SECOND * seconds));

	if (raster_b != NULL){
		raster_b->cartesian_blk_mm = cartesian_mm/float(steps);
	}

	//SERIAL_ECHOPGM("mm="); SERIAL_ECHOLN(cartesian_mm);
	//SERIAL_ECHOPGM(" seconds="); SERIAL_ECHOLN(seconds);
	//SERIAL_ECHOPGM(" steps="); SERIAL_ECHOLN(steps);
	//SERIAL_ECHOPGM(" feedrate="); SERIAL_ECHOLN(myfeedrate);

	for (int s = 1; s <= steps; s++) {
		float fraction = float(s) / float(steps);
		for(int8_t i=0; i < NUM_AXIS; i++) {
			destination[i] = current_position[i] + difference[i] * fraction;
		}

		calculate_delta(destination);
#ifdef NONLINEAR_BED_LEVELING
		adjust_delta(destination);
#endif

		plan_buffer_line(delta[X_AXIS], delta[Y_AXIS], delta[Z_AXIS],
			destination[E_AXIS], myfeedrate*feedmultiply/60/100.0,
			active_extruder,raster_b);
	}

	for(int8_t i=0; i < NUM_AXIS; i++) {
		current_position[i] = destination[i];
		machine_position[i] = current_position[i] + tool_config[active_tool].tool_offset[i];
	}
}

void prepare_arc_move(char isclockwise) {
	float r = hypot(offset[X_AXIS], offset[Y_AXIS]); // Compute arc radius for mc_arc

	// Trace the arc
	mc_arc(current_position, destination, offset, X_AXIS, Y_AXIS, Z_AXIS, feedrate*feedmultiply/60/100.0, r, isclockwise, active_extruder);

	// As far as the parser is concerned, the position is now == target. In reality the
	// motion control system might still be processing the action and the real tool position
	// in any intermediate location.
	for(int8_t i=0; i < NUM_AXIS; i++) {
		current_position[i] = destination[i];
		machine_position[i] = current_position[i] + tool_config[active_tool].tool_offset[i];
	}
	previous_millis_cmd = millis();
}

#if defined(CONTROLLERFAN_PIN) && CONTROLLERFAN_PIN > -1

#if defined(FAN_PIN)
#if CONTROLLERFAN_PIN == FAN_PIN
#error "You cannot set CONTROLLERFAN_PIN equal to FAN_PIN"
#endif
#endif

unsigned long lastMotor = 0; //Save the time for when a motor was turned on last
unsigned long lastMotorCheck = 0;

void controllerFan()
{
	if ((millis() - lastMotorCheck) >= 2500) //Not a time critical function, so we only check every 2500ms
	{
		lastMotorCheck = millis();

		if(!READ(X_ENABLE_PIN) || !READ(Y_ENABLE_PIN) || !READ(Z_ENABLE_PIN) || (soft_pwm_bed > 0)
#if EXTRUDERS > 2
			|| !READ(E2_ENABLE_PIN)
#endif
#if EXTRUDER > 1
#if defined(X2_ENABLE_PIN) && X2_ENABLE_PIN > -1
			|| !READ(X2_ENABLE_PIN)
#endif
			|| !READ(E1_ENABLE_PIN)
#endif
			|| !READ(E0_ENABLE_PIN)) //If any of the drivers are enabled...
		{
			lastMotor = millis(); //... set time to NOW so the fan will turn on
		}

		if ((millis() - lastMotor) >= (CONTROLLERFAN_SECS*1000UL) || lastMotor == 0) //If the last time any driver was enabled, is longer since than CONTROLLERSEC...
		{
			digitalWrite(CONTROLLERFAN_PIN, 0);
			analogWrite(CONTROLLERFAN_PIN, 0);
		}
		else
		{
			// allows digital or PWM fan output to be used (see M42 handling)
			digitalWrite(CONTROLLERFAN_PIN, CONTROLLERFAN_SPEED);
			analogWrite(CONTROLLERFAN_PIN, CONTROLLERFAN_SPEED);
		}
	}
}
#endif

#ifdef TEMP_STAT_LEDS
static bool blue_led = false;
static bool red_led = false;
static uint32_t stat_update = 0;

void handle_status_leds(void) {
	float max_temp = 0.0;
	if(millis() > stat_update) {
		stat_update += 500; // Update every 0.5s
		for (int8_t cur_extruder = 0; cur_extruder < EXTRUDERS; ++cur_extruder) {
			max_temp = max(max_temp, degHotend(cur_extruder));
			max_temp = max(max_temp, degTargetHotend(cur_extruder));
		}
#if defined(TEMP_BED_PIN) && TEMP_BED_PIN > -1
		max_temp = max(max_temp, degTargetBed());
		max_temp = max(max_temp, degBed());
#endif
		if((max_temp > 55.0) && (red_led == false)) {
			digitalWrite(STAT_LED_RED, 1);
			digitalWrite(STAT_LED_LASER_BLUE, 0);
			red_led = true;
			blue_led = false;
		}
		if((max_temp < 54.0) && (blue_led == false)) {
			digitalWrite(STAT_LED_RED, 0);
			digitalWrite(STAT_LED_LASER_BLUE, 1);
			red_led = false;
			blue_led = true;
		}
	}
}
#endif

void manage_inactivity()
{	
	if (MSerial.overflow){
		SERIAL_PROTOCOLPGM("IRQ ");
		SERIAL_PROTOCOLLNPGM(MSG_OVERFLOW);
		kill();
	}

	if( (millis() - previous_millis_cmd) >  max_inactive_time )
		if(max_inactive_time)
			kill();
	if(stepper_inactive_time)  {
		if( (millis() - previous_millis_cmd) >  stepper_inactive_time )
		{
			if(blocks_queued() == false) {
				disable_x();
				disable_y();
				disable_z();
				disable_e0();
				disable_e1();
				disable_e2();
			}
		}
	}
#if defined(KILL_PIN) && KILL_PIN > -1
	if( 0 == READ(KILL_PIN) )
		kill();
#endif
#if defined(CONTROLLERFAN_PIN) && CONTROLLERFAN_PIN > -1
	controllerFan(); //Check if fan should be turned on to cool stepper drivers down
#endif
#ifdef EXTRUDER_RUNOUT_PREVENT
	if( (millis() - previous_millis_cmd) >  EXTRUDER_RUNOUT_SECONDS*1000 )
		if(degHotend(active_extruder)>EXTRUDER_RUNOUT_MINTEMP)
		{
			bool oldstatus=READ(E0_ENABLE_PIN);
			enable_e0();
			float oldepos=current_position[E_AXIS];
			float oldedes=destination[E_AXIS];
			plan_buffer_line(destination[X_AXIS], destination[Y_AXIS], destination[Z_AXIS],
				destination[E_AXIS]+EXTRUDER_RUNOUT_EXTRUDE*EXTRUDER_RUNOUT_ESTEPS/axis_steps_per_unit[E_AXIS],
				EXTRUDER_RUNOUT_SPEED/60.*EXTRUDER_RUNOUT_ESTEPS/axis_steps_per_unit[E_AXIS], active_extruder,NULL);
			current_position[E_AXIS]=oldepos;
			destination[E_AXIS]=oldedes;
			plan_set_e_position(oldepos);
			previous_millis_cmd=millis();
			st_synchronize();
			WRITE(E0_ENABLE_PIN,oldstatus);
		}
#endif
#ifdef TEMP_STAT_LEDS
		handle_status_leds();
#endif
		check_axes_activity();
}

void kill()
{
	//cli(); // Stop interrupts
	disable_heater();

	disable_x();
	disable_y();
	disable_z();
	disable_e0();
	disable_e1();
	disable_e2();

#if defined(PS_ON_PIN) && PS_ON_PIN > -1
	pinMode(PS_ON_PIN,INPUT);
#endif
	SERIAL_ERROR_START;
	SERIAL_ERRORLNPGM(MSG_ERR_KILLED);

	suicide();
	while(1) { /* Intentionally left empty */ } // Wait for reset
}

void Stop()
{
	disable_heater();
	if(Stopped == false) {
		Stopped = true;
		SERIAL_ERROR_START;
		SERIAL_ERRORLNPGM(MSG_ERR_STOPPED);
	}
}

bool IsStopped() { return Stopped; };

#ifdef FAST_PWM_FAN
void setPwmFrequency(uint8_t pin, int val)
{
	val &= 0x07;
	switch(digitalPinToTimer(pin))
	{

#if defined(TCCR0A)
	case TIMER0A:
	case TIMER0B:
		//         TCCR0B &= ~(_BV(CS00) | _BV(CS01) | _BV(CS02));
		//         TCCR0B |= val;
		break;
#endif

#if defined(TCCR1A)
	case TIMER1A:
	case TIMER1B:
		//         TCCR1B &= ~(_BV(CS10) | _BV(CS11) | _BV(CS12));
		//         TCCR1B |= val;
		break;
#endif

#if defined(TCCR2)
	case TIMER2:
	case TIMER2:
		TCCR2 &= ~(_BV(CS10) | _BV(CS11) | _BV(CS12));
		TCCR2 |= val;
		break;
#endif

#if defined(TCCR2A)
	case TIMER2A:
	case TIMER2B:
		TCCR2B &= ~(_BV(CS20) | _BV(CS21) | _BV(CS22));
		TCCR2B |= val;
		break;
#endif

#if defined(TCCR3A)
	case TIMER3A:
	case TIMER3B:
	case TIMER3C:
		TCCR3B &= ~(_BV(CS30) | _BV(CS31) | _BV(CS32));
		TCCR3B |= val;
		break;
#endif

#if defined(TCCR4A)
	case TIMER4A:
	case TIMER4B:
	case TIMER4C:
		TCCR4B &= ~(_BV(CS40) | _BV(CS41) | _BV(CS42));
		TCCR4B |= val;
		break;
#endif

#if defined(TCCR5A)
	case TIMER5A:
	case TIMER5B:
	case TIMER5C:
		TCCR5B &= ~(_BV(CS50) | _BV(CS51) | _BV(CS52));
		TCCR5B |= val;
		break;
#endif

	}
}
#endif //FAST_PWM_FAN

bool setTargetedHotend(int code){
	tmp_extruder = active_extruder;
	if(code_seen('T')) {
		tmp_extruder = code_value();
		if(tmp_extruder >= EXTRUDERS) {
			SERIAL_ECHO_START;
			switch(code){
			case 104:
				SERIAL_ECHO(MSG_M104_INVALID_EXTRUDER);
				break;
			case 105:
				SERIAL_ECHO(MSG_M105_INVALID_EXTRUDER);
				break;
			case 109:
				SERIAL_ECHO(MSG_M109_INVALID_EXTRUDER);
				break;
			case 218:
				SERIAL_ECHO(MSG_M218_INVALID_EXTRUDER);
				break;
			}
			SERIAL_ECHOLN(tmp_extruder);
			return true;
		}
	}
	return false;
}

// Delays variable defined milliseconds. Compiler compatibility fix for _delay_ms(),
// which only accepts constants in future compiler releases.
void delay_ms(uint16_t ms) 
{
	while ( ms-- ) { _delay_ms(1); }
}


// Delays variable defined microseconds. Compiler compatibility fix for _delay_us(),
// which only accepts constants in future compiler releases. Written to perform more 
// efficiently with larger delays, as the counter adds parasitic time in each iteration.
void delay_us(uint32_t us) 
{
	while (us) {
		if (us < 10) { 
			_delay_us(1);
			us--;
		} else if (us < 100) {
			_delay_us(10);
			us -= 10;
		} else if (us < 1000) {
			_delay_us(100);
			us -= 100;
		} else {
			_delay_ms(1);
			us -= 1000;
		}
	}
}
